@@ -1,27 +1,53 @@
+## ----setup, include=FALSE-----------------------------------------------------
+knitr::opts_chunk$set(dpi=72)
+
 ## ----pack, warning = FALSE, message = FALSE-----------------------------------
 library( celltrackR )
 library( ggplot2 )
 
-## ---- echo = FALSE------------------------------------------------------------
+## ----echo = FALSE-------------------------------------------------------------
 # Save current par() settings
 oldpar <- par( no.readonly =TRUE )
 
-## ----Tdata--------------------------------------------------------------------
-str( TCells, list.len = 3 )
+## -----------------------------------------------------------------------------
+str( TCells, list.len = 1 )
 
 ## -----------------------------------------------------------------------------
-head( TCells[[1]] )
+load( system.file("extdata", "TCellsRaw.rda", package="celltrackR" ) )
+load( system.file("extdata", "BCellsRaw.rda", package="celltrackR" ) )
+
+## ----Tdata--------------------------------------------------------------------
+str( TCellsRaw, list.len = 1 )
+
+## -----------------------------------------------------------------------------
+head( TCellsRaw[[1]] )
+
+## -----------------------------------------------------------------------------
+# for reproducibility
+set.seed(2021)
+
+# sample names, sort them in numeric order, and use them to subset the tracks
+Bsample <- sample( names(BCellsRaw),50)
+Bsample <- Bsample[ order(as.integer(Bsample)) ]
+BCellsRaw <- BCellsRaw[ Bsample ]
+
+# same for the T cells, but now we include a few specific IDs we'll need for the
+# analysis below.
+Tsample <- sample( names(TCellsRaw),44)
+Tsample <- unique( c( Tsample, "5","9658","83","6080","7832","8352" ) )
+Tsample <- Tsample[ order(as.integer(Tsample)) ]
+TCellsRaw <- TCellsRaw[ Tsample ]
 
 ## ----tracklength-check--------------------------------------------------------
 # Each track has a coordinate matrix with one row per coordinate;
 # The number of steps is the number of rows minus one.
-track.lengths <- sapply( TCells, nrow ) - 1
-hist( track.lengths, xlab = "Track length (#steps)" )
+track.lengths <- sapply( TCellsRaw, nrow ) - 1
+hist( track.lengths, xlab = "Track length (#steps)", breaks = seq(0,40 ) )
 summary( track.lengths )
 
 ## ----max-tracklength----------------------------------------------------------
 # This is the number of coordinates, so the number of steps is one less.
-maxTrackLength( TCells )
+maxTrackLength( TCellsRaw )
 
 ## ----filter-short-------------------------------------------------------------
 # nrow() of a track is always the number of steps plus one.
@@ -30,24 +56,21 @@ filter.criterion <- function( x, min.steps ){
   nrow(x) > min.steps
 }
 
-TCells.filtered <- filterTracks( filter.criterion, TCells, min.steps = 11 )
-# Or shorthand: filterTracks( function(x) nrow(x) > min.steps, TCells )
+TCells.filtered <- filterTracks( filter.criterion, TCellsRaw, min.steps = 11 )
+# Or shorthand: filterTracks( function(x) nrow(x) > min.steps, TCellsRaw )
 
-## ----check-filtered, fig.width=6----------------------------------------------
+## ----check-filtered-----------------------------------------------------------
 # Find lengths of the filtered dataset
 track.lengths.filtered <- sapply( TCells.filtered, nrow ) - 1
 
 # Histograms of track lengths before and after
-par( mfrow=c(1,2) )
-hist( track.lengths, xlab = "Track length (#steps)",
-      main = "Before filtering", breaks = seq(0, 40, by = 10 ) )
-hist( track.lengths.filtered, xlab = "Track length (#steps)",
-      main = "After filtering",  breaks = seq(0, 40, by = 10 ))
+c( "min length before filter" = min( track.lengths ),
+   "min length after filter" = min( track.lengths.filtered ) )
 
 # Check how many tracks are left:
 length( TCells.filtered )
 
-## ---- fig.width=7-------------------------------------------------------------
+## ----fig.width=7, fig.height = 3.5--------------------------------------------
 # Drift is 0.05 micron/sec in each dimension
 drift.speed <- c( 0.05, 0.05, 0.05 )
 add.drift <- function( x, drift.vector )
@@ -67,32 +90,46 @@ add.drift <- function( x, drift.vector )
 }
 
 # Create data with drift
-TCells.drift <- as.tracks( lapply( TCells, add.drift, drift.speed ) )
+TCells.drift <- as.tracks( lapply( TCellsRaw, add.drift, drift.speed ) )
 
-# Plot both for comparison
+# Plot tracks and 'rose plot' with overlayed starting point
 par(mfrow=c(1,2) )
-plot(TCells, main = "original data" )
-plot(TCells.drift, main = "with tissue drift" )
-
-# Overlay starting points to view directionality
-plot( normalizeTracks(TCells), main = "original data" )
-plot( normalizeTracks(TCells.drift), main = "with tissue drift" )
+plot(TCells.drift, main = "drift", cex = 0 )
+plot( normalizeTracks(TCells.drift), main = "drift (rose plot)", cex = 0, col = "gray" )
+abline( h = 0 )
+abline( v = 0 )
 
 
-## -----------------------------------------------------------------------------
-hotellingsTest( TCells, plot = TRUE, col = "gray" )
+## ----fig.width = 4, fig.height = 3--------------------------------------------
+hotellingsTest( TCellsRaw, col = "gray" )
 
-## -----------------------------------------------------------------------------
-hotellingsTest( TCells, plot = TRUE, col = "gray", step.spacing = 5 )
+## ----fig.width = 6, fig.height = 6--------------------------------------------
+hotellingsTest( BCellsRaw, col = "gray" )
 
-## -----------------------------------------------------------------------------
-hotellingsTest( TCells.drift, plot = TRUE, col = "gray", step.spacing = 5 )
+## ----fig.width = 4, fig.height = 3.5------------------------------------------
+# Compute autocovariance, normalize so the first point lies at 1
+Tacov <- aggregate( TCellsRaw, overallDot )
+Tacov$value <- Tacov$value / Tacov$value[1]
 
-## -----------------------------------------------------------------------------
-hotellingsTest( TCells, dim = c("x","y","z"), step.spacing = 5 )
-hotellingsTest( TCells.drift, dim = c("x","y","z"), step.spacing = 5 )
+# The same for B cells:
+Bacov <- aggregate( BCellsRaw, overallDot )
+Bacov$value <- Bacov$value / Bacov$value[1]
 
-## ---- echo = FALSE, fig.width=7-----------------------------------------------
+# Compare autocovariances:
+plot( Tacov )
+points( Bacov, col = "red" )
+
+## ----fig.width = 6, fig.height = 6--------------------------------------------
+hotellingsTest( BCellsRaw, col = "gray", step.spacing = 10 )
+
+## ----fig.width = 4, fig.height = 4--------------------------------------------
+hotellingsTest( TCells.drift, plot = TRUE, col = "gray", step.spacing = 10 )
+
+## ----fig.width = 6, fig.height = 6--------------------------------------------
+hotellingsTest( TCellsRaw, dim = c("x","y","z"), step.spacing = 10 )
+hotellingsTest( TCells.drift, dim = c("x","y","z"), step.spacing = 10 )
+
+## ----echo = FALSE, fig.width=7------------------------------------------------
 # sp <- seq(0,10)
 # 
 # htest.nodrift <- lapply( sp, function(x) hotellingsTest( TCells, 
@@ -132,15 +169,15 @@ hotellingsTest( TCells.drift, dim = c("x","y","z"), step.spacing = 5 )
 # 
 # gridExtra::grid.arrange(p1,p2,ncol=2)
 
-## ---- warning = FALSE, message = FALSE, fig.width=7---------------------------
+## ----cellPairs, warning = FALSE, message = FALSE, fig.width=6, fig.heigth = 2.5----
 # compute for both original as drift data
 df.drift <- analyzeCellPairs( TCells.drift )
-df.norm <- analyzeCellPairs( TCells )
+df.norm <- analyzeCellPairs( TCellsRaw )
 
 # Plot
 p.norm <- ggplot( df.norm, aes( x = dist, y = angle ) ) +
-  geom_point( color = "gray40" ) +
-  stat_smooth( span = 1, color = "black" )+
+  geom_point( color = "gray70", size = 0.5 ) +
+  stat_smooth( span = 1, fill = "blue" )+
   labs( x = "distance between cell pairs",
         y = "angle between cell pairs",
         title = "original data") +
@@ -148,8 +185,8 @@ p.norm <- ggplot( df.norm, aes( x = dist, y = angle ) ) +
   theme_classic()
 
 p.drift <- ggplot( df.drift, aes( x = dist, y = angle ) ) +
-  geom_point( color = "gray40" ) +
-  stat_smooth( span = 1, color = "black" )+
+  geom_point( color = "gray70", size = 0.5 ) +
+  stat_smooth( span = 1, fill = "blue" )+
   labs( x = "distance between cell pairs",
         y = "angle between cell pairs",
         title = "data with drift") +
@@ -158,15 +195,15 @@ p.drift <- ggplot( df.drift, aes( x = dist, y = angle ) ) +
 
 gridExtra::grid.arrange( p.norm, p.drift, ncol = 2 )
 
-## ---- warning = FALSE, message = FALSE, fig.width=7---------------------------
-# compute for both original as drift data
+## ----stepPairs, warning = FALSE, message = FALSE, fig.width=6, fig.heigth = 2.5----
+# compute for both original as drift data.
 df.drift <- analyzeStepPairs( TCells.drift, filter.steps = function(x) displacement(x)>2  )
-df.norm <- analyzeStepPairs( TCells, filter.steps = function(x) displacement(x)>2  )
+df.norm <- analyzeStepPairs( TCellsRaw, filter.steps = function(x) displacement(x)>2  )
 
 # Plot
 p.norm <- ggplot( df.norm, aes( x = dist, y = angle ) ) +
-  geom_point( color = "gray40", size = 0.5 ) +
-  stat_smooth( span = 0.75, color = "black" )+
+  geom_point( color = "gray70", size = 0.5 ) +
+  stat_smooth( span = 0.75, fill="blue" )+
   labs( x = "distance between step pairs",
         y = "angle between step pairs",
         title = "original data") +
@@ -174,8 +211,8 @@ p.norm <- ggplot( df.norm, aes( x = dist, y = angle ) ) +
   theme_classic()
 
 p.drift <- ggplot( df.drift, aes( x = dist, y = angle ) ) +
-  geom_point( color = "gray40", size = 0.5 ) +
-  stat_smooth( span = 0.75, color = "black" )+
+  geom_point( color = "gray70", size = 0.5 ) +
+  stat_smooth( span = 0.75,  fill="blue")+
   labs( x = "distance between step pairs",
         y = "angle between step pairs",
         title = "data with drift") +
@@ -196,7 +233,7 @@ mean.displacement <- colMeans( step.disp )
 drift.speed <- mean.displacement/timeStep( TCells.drift )
 drift.speed
 
-## ---- fig.width = 7-----------------------------------------------------------
+## ----fig.width = 6, fig.height = 3.5------------------------------------------
 correct.drift <- function( x, drift.vector )
 {
   # separate timepoints and coordinates
@@ -216,23 +253,17 @@ correct.drift <- function( x, drift.vector )
 # Create data with drift
 TCells.corrected <- as.tracks( lapply( TCells.drift, correct.drift, drift.speed ) )
 
-# Compare
+# Compare (zoom in on part of the field to see better)
 par( mfrow = c(1,2) )
-plot( TCells, col = "gray", main = "uncorrected" )
-plot( TCells.drift, col = "red", add = TRUE )
-plot( TCells, col = "gray", main = "corrected" )
-plot( TCells.corrected, col = "red", add = TRUE )
+plot( TCellsRaw, col = "blue", main = "uncorrected", cex = 0, pch.start = NA, xlim = c(200,400), ylim = c(0,200) )
+plot( TCells.drift, col = "red", add = TRUE, cex = 0, pch.start = NA )
+plot( TCellsRaw, col = "blue", main = "corrected", cex = 0, pch.start = NA, xlim = c(200,400), ylim = c(0,200)  )
+plot( TCells.corrected, col = "red", add = TRUE, cex = 0, pch.start = NA  )
 
-#
-par( mfrow = c(1,2) )
-plot( normalizeTracks(TCells), col = "gray", main = "uncorrected" )
-plot( normalizeTracks(TCells.drift), col = "red", add = TRUE )
-plot( normalizeTracks(TCells), col = "gray", main = "corrected" )
-plot( normalizeTracks(TCells.corrected), col = "red", add = TRUE )
 
 ## -----------------------------------------------------------------------------
-# Take the track with id "2"
-dup.track <- TCells[["2"]]
+# Take the track with id "5"
+dup.track <- TCellsRaw[["5"]]
 
 # Add some noise to coordinates
 dup.track[,"x"] <- dup.track[,"x"] + rnorm( nrow(dup.track), sd = 0.5 )
@@ -242,38 +273,46 @@ dup.track[,"z"] <- dup.track[,"z"] + rnorm( nrow(dup.track), sd = 0.5 )
 # Wrap the track in a tracks object and add it to the TCell data with
 # a unique id number
 dup.track <- wrapTrack( dup.track )
-names(dup.track) <- "22"
-TCells.dup <- c( TCells, dup.track )
+new.id <- max( as.numeric( names( TCellsRaw ) ) ) + 1
+names(dup.track) <- as.character( new.id )
+TCells.dup <- c( TCellsRaw, dup.track )
 
-## ---- warning = FALSE---------------------------------------------------------
+## ----warning = FALSE, fig.width = 3, fig.height = 2.5-------------------------
 df <- analyzeCellPairs( TCells.dup )
 
 # label cellpairs that have both angle and distance below threshold
-angle.thresh <- 90 # in degrees
+angle.thresh <- 5 # in degrees
 dist.thresh <- 10 # this should be the expected cell radius
 df$id <- paste0( df$cell1,"-",df$cell2 )
 df$id[ !(df$angle < angle.thresh & df$dist < dist.thresh) ] <- "" 
 
-# Plot
+# Plot; zoom in on the region with small angles and distances
 ggplot( df, aes( x = dist, y = angle ) ) +
   geom_point( color = "gray40" ) +
-  geom_text( aes( label = id ), color = "red" ) +
+  geom_text( aes( label = id ), color = "red", hjust = -0.1 ) +
   labs( x = "distance between cell pairs",
         y = "angle between cell pairs" ) +
+  coord_cartesian( xlim=c(0,30), ylim=c(0,20) ) +
   geom_hline( yintercept = angle.thresh, col = "blue",lty=2 ) +
   geom_vline( xintercept = dist.thresh, col = "blue", lty=2) +
   theme_classic()
 
 
-## -----------------------------------------------------------------------------
-plot( TCells.dup[c("2","22")])
+## ----fig.width = 7, fig.height = 2.5------------------------------------------
+par( mfrow=c(1,3))
+plot( TCells.dup[c("5","9659") ] )
+plot( TCells.dup[c("83","6080") ] )
+plot( TCells.dup[c("7832","8352") ] )
 
 ## -----------------------------------------------------------------------------
-tracks <- TCells
+corrected.dup <- TCells.dup[ names( TCells.dup ) != "9659" ]
+
+## -----------------------------------------------------------------------------
+tracks <- TCellsRaw
 bb <- boundingBox( tracks )
 bb
 
-## -----------------------------------------------------------------------------
+## ----fig.width = 3, fig.height = 2--------------------------------------------
 # Define points:
 lower1 <- c( bb["min","x"], bb["min","y"], bb["min","z"] )
 lower2 <- c( bb["max","x"], bb["min","y"], bb["min","z"] )
@@ -292,22 +331,29 @@ df <- data.frame( angles = angles,
 # Plot
 ggplot( df, aes( x = distances,
                  y = angles ) ) +
-  geom_point( color = "gray40" ) +
-  stat_smooth( method = "loess", span = 1, color = "black" ) +
+  geom_point( color = "gray70", size = 0.5 ) +
+  stat_smooth( method = "loess", span = 1, fill="blue" ) +
   geom_hline( yintercept = 32.7, color = "red" ) +
   scale_x_continuous( limits=c(0,zsize ), expand = c(0,0) ) +
   theme_classic()
 
+## ----fig.width = 4, fig.height = 3--------------------------------------------
+par( mar = c(5, 4, 0.5, 2) + 0.1 )
+plot( TCellsRaw, dims = c("x","z" ) )
+
+## -----------------------------------------------------------------------------
+boundingBox( TCellsRaw )
+
 ## ----get-steps----------------------------------------------------------------
 # Extract all subtracks of length 1 (that is, all "steps")
-single.steps <- subtracks( TCells, 1 )
+single.steps <- subtracks( TCellsRaw, 1 )
 
 # The output is a new tracks object with a unique track for each step
 # in the data (no longer grouped by the original cell they came from):
 str( single.steps, list.len = 3 )
 
 ## ----check-avdt---------------------------------------------------------------
-median.dt <- timeStep( TCells )
+median.dt <- timeStep( TCellsRaw )
 median.dt
 
 ## ----check-dt-----------------------------------------------------------------
@@ -318,6 +364,13 @@ str(step.dt)
 dt.diff.perc <- (step.dt - median.dt) * 100 / median.dt
 hist( dt.diff.perc, xlab = "dt (percentage difference from median)" )
 
+## -----------------------------------------------------------------------------
+range( step.dt )
+unique( step.dt )
+
+## -----------------------------------------------------------------------------
+sum( step.dt == 48 )
+
 ## -----------------------------------------------------------------------------
 # This function randomly removes coordinates from a track dataset with probability "prob"
 remove.points <- function( track, prob=0.1 ){
@@ -330,21 +383,11 @@ remove.points <- function( track, prob=0.1 ){
 }
 
 # Apply function to dataset to randomly remove coordinates in the data
-TCells.gap <- as.tracks( lapply( TCells, remove.points ) )
+TCells.gap <- as.tracks( lapply( TCellsRaw, remove.points ) )
 
 ## -----------------------------------------------------------------------------
-# median dt of the new data
-median.dt.gap <- timeStep( TCells.gap )
-
 # duration of the individual steps
 steps.gap <- subtracks( TCells.gap, 1 )
-step.dt.gap <- sapply( steps.gap, duration )
-
-# express difference as percentage of median dt
-dt.diff.perc.gap <- (step.dt.gap - median.dt.gap) * 100 / median.dt.gap
-hist( dt.diff.perc.gap, xlab = "dt (percentage difference from median)" )
-
-## -----------------------------------------------------------------------------
 
 T1.step.disp <- sapply( single.steps, displacement )
 T1.gap.disp <- sapply( steps.gap, displacement )
@@ -356,7 +399,7 @@ T1.norm.disp <- sapply( single.steps, normalizeToDuration( displacement ) )
 T1.norm.gap.disp <- sapply( steps.gap, normalizeToDuration( displacement ) )
 lapply( list( original = T1.norm.disp, gaps = T1.norm.gap.disp ), summary )
 
-## ---- fig.width=6-------------------------------------------------------------
+## ----fig.width=6--------------------------------------------------------------
 
 # Repair gaps by splitting or interpolation, the number of tracks is 
 # different after each fix
@@ -367,11 +410,11 @@ c( "after splitting" = length( split.gap),
    "after interpolation" = length( interpolate.gap ) )
 
 ## -----------------------------------------------------------------------------
-T2 <- subsample( TCells, k = 2 )
+T2 <- subsample( TCellsRaw, k = 2 )
 
 ## -----------------------------------------------------------------------------
 # displacement
-T1.steps <- subtracks( TCells, 1 )
+T1.steps <- subtracks( TCellsRaw, 1 )
 T1.disp <- sapply( T1.steps, displacement )
 
 T2.steps <- subtracks( T2, 1 )
@@ -382,7 +425,7 @@ lapply( list( T1 = T1.disp, T2 = T2.disp ), summary )
 
 ## -----------------------------------------------------------------------------
 # interpolate both datasets at the time resolution of the neutrophils
-dt <- timeStep( TCells )
+dt <- timeStep( TCellsRaw )
 interpolate.dt <- function( x, dt, how = "spline" ){
   trange <- range( timePoints( wrapTrack( x ) ) )
   tvec <- seq( trange[1], trange[2], by = dt )
@@ -390,7 +433,7 @@ interpolate.dt <- function( x, dt, how = "spline" ){
   return(x)
 }
 
-T1.corrected <- as.tracks( lapply( TCells, interpolate.dt, dt = dt ) )
+T1.corrected <- as.tracks( lapply( TCellsRaw, interpolate.dt, dt = dt ) )
 T2.corrected <- as.tracks( lapply( T2, interpolate.dt, dt = dt ) )
 
 # Check the effect on the displacement statistics:
@@ -404,7 +447,111 @@ lapply( list( T1 = T1.disp, T2 = T2.disp,
               T1.corr = T1.corr.disp, T2.corr = T2.corr.disp ), 
         summary )
 
-## ---- echo = FALSE------------------------------------------------------------
+## ----echo = FALSE-------------------------------------------------------------
 # Reset par() settings
 par(oldpar)
 
+## ----dpi = 100----------------------------------------------------------------
+load( system.file("extdata", "TCellsRaw.rda", package="celltrackR" ) )
+
+par( mar = c(2, 2, 1, 1) + 0.1 )
+plot( TCellsRaw )
+
+## -----------------------------------------------------------------------------
+cellSpeeds <- sapply( TCellsRaw, speed )
+hist( cellSpeeds, breaks = 30 )
+
+## -----------------------------------------------------------------------------
+cutoff <- 0.1 # 0.1 micron/sec = 6 micron/min
+loSpeed <- filterTracks( function(t) speed(t) < cutoff, TCellsRaw )
+hiSpeed <- filterTracks( function(t) speed(t) >= cutoff, TCellsRaw )
+
+## ----fig.width = 6, fig.height = 3.5------------------------------------------
+par(mfrow=c(1,2))
+plot( loSpeed, main = "Below cutoff" )
+plot( hiSpeed, main = "Above cutoff" )
+
+## ----bic-nonmotile------------------------------------------------------------
+bicNonMotile <- function( track, sigma ){
+  
+  # we'll use only x and y coordinates since we saw earlier that the z-dimension was
+  # not so reliable
+  allPoints <- track[,c("x","y")]
+  
+  # Compute the log likelihood under a multivariate gaussian.
+  # For each point, we get the density under the Gaussian distribution 
+  # (using dmvnorm from the mvtnorm package).
+  # The product of these densities is then the likelihood; but since we need the 
+  # log likelihood, we can also first log-transform and then sum:
+  Lpoints <- mvtnorm::dmvnorm( allPoints, 
+                      mean = colMeans(allPoints), # for a Gaussian around the mean position
+                      sigma = sigma*diag(2), # sd of the Gaussian (which we should choose)
+                      log = TRUE )
+  logL <- sum( Lpoints )
+  
+  # BIC = k log n - 2 logL; here k = 3 ( mean x, mean y, sigma )
+  return( 3*log(nrow(allPoints)) - 2*logL )
+}
+
+## -----------------------------------------------------------------------------
+# the BIC for a given cutoff m
+bicAtCutoff <- function( track, m, sigma ){
+  
+  # we'll use only x and y coordinates since we saw earlier that the z-dimension was
+  # not so reliable
+  allPoints <- track[,c("x","y")]
+  
+  # Split into two coordinate sets based on the cutoff m:
+  firstCoords <- allPoints[1:m, , drop = FALSE]
+  lastCoords <- allPoints[(m+1):nrow(allPoints), , drop = FALSE ]
+  
+  # Compute log likelihood under two separate Gaussians:
+  Lpoints1 <- mvtnorm::dmvnorm( firstCoords, 
+    mean = colMeans(firstCoords), 
+    sigma = sigma*diag(2), 
+    log = TRUE )
+  Lpoints2 <- mvtnorm::dmvnorm( lastCoords, 
+    mean = colMeans(lastCoords), 
+    sigma = sigma*diag(2), 
+    log = TRUE )
+  logL <- sum( Lpoints1 ) + sum( Lpoints2 )
+  
+  # BIC = k log n - 2 logL; here k = 6 ( 2*mean x, 2*mean y, sigma, and m )
+  return( 6*log(nrow(allPoints)) - 2*logL )
+}
+
+# We'll try all possible cutoffs m, and choose best model (minimal BIC)
+# to compare to our non-motile "null hypothesis":
+bicMotile <- function( track, sigma ){
+  
+  # cutoff anywhere from after the first two coordinates to 
+  # before the last two (we want at least 2 points in each Gaussian,
+  # to prevent fitting of a single point)
+  cutoffOptions <- 2:(nrow(track)-2)
+  
+ 	min( sapply( cutoffOptions, function(m) bicAtCutoff(track,m,sigma) ) )
+}
+
+## ----fig.width = 6, fig.height = 3.5------------------------------------------
+deltaBIC <- function( x, sigma ){
+  b1 <- bicNonMotile( x, sigma )
+  b2 <- bicMotile( x, sigma )
+  d <- b1 - b2
+  d
+}
+
+dBIC <- sapply( TCellsRaw, deltaBIC, 7 )
+motile <- TCellsRaw[ dBIC >= 6 ]
+nonMotile <- TCellsRaw[ dBIC < 6 ]
+
+
+# Plot again for comparison:
+par(mfrow=c(1,2))
+plot( motile, main = "Motile" )
+plot( nonMotile, main = "Non-Motile" )
+
+## ----fig.width = 4, fig.height = 3--------------------------------------------
+plot( dBIC, cellSpeeds, xlim = c(0,20) )
+abline( v = 6, col = "red" ) # delta BIC cutoff
+abline( h = max(cellSpeeds), col = "blue", lty = 2 ) # max speed of all cells
+
